import numpy as np
import os, glob
import matplotlib.pyplot as plt
from obspy import read, Trace
from mydetails import info


def paths(type='*.SAC'):
    """
    Get the paths
    :param type: Filetype
    :return:     the path to the waveforms, the path to the visual classifications and the path to the subsets
    """
    _, path_to_folder, project,_, path_waveforms = info()
    path_postprocessed = path_to_folder+'/waveforms/'+type
    path_classifications_hand = path_to_folder+'/classified/'
    path_subsets = path_to_folder+'/subsets/'
    return path_postprocessed, path_classifications_hand, path_subsets, path_waveforms


class ostream(object):
    """
    Class for reading Seismic data and storing them in an obspy stream
    """
    def __init__(self, originpath, number_of_distance_classes=0, constraint = None):
        self.constraint = constraint
        self.n = number_of_distance_classes
        self.reader(originpath)
        self.stream_ = self.stream_original.copy()

    def reader(self, path, normalize =True):
        self.corrlist = []
        i = 0
        if self.constraint == None:
            for corr in glob.glob(path): 
                if i == 0: 
                    stream_ = read(corr)
                else: 
                    trace = read(corr)
                    stream_ = stream_.__add__(trace)
                self.corrlist.append(os.path.basename(corr))
                i+=1
        else: 
            for i in range(len(self.constraint)): 
                if i == 0: 
                    stream_ = read(path+str(self.constraint[i]))
                else: 
                    trace = read(path+str(self.constraint[i]))
                    stream_ = stream_.__add__(trace)
                self.corrlist.append(self.constraint[i])
                i+=1

        if normalize == True:
            stream_ = stream_.normalize(global_max=False)
        self.stream_original = stream_

    def normalize(self, type='absolute'): 
        if type == 'positive_max': 
            for trace in self.stream_: 
                trace.data = trace.data/np.max(trace.data)
        elif type == 'absolute':  
            self.stream_ = self.stream_.normalize(global_max=False)
        else: 
            print('No valid Normalization chosen, please revisit. \nUsable options: absolute or positive_max')

    def distance_attribute(self): 
        try: 
            for trace in self.stream_: 
                trace.stats.distance = trace.stats.sac.dist
        except: 
            print("COULD NOT OBTAIN DISTANCE INFORMATION! \n MAYBE DUE TO FILETYPE (only .SAC supported)")

    def get_max_dist(self): 
        tmp = 0
        for trace in self.stream_: 
            if trace.stats.sac.dist > tmp: 
                tmp = trace.stats.sac.dist
        self.maxdist = tmp

    def get_time(self): 
        l = np.arange(len(self.stream_[0].data)/2+1)
        l = np.append(-1*l[-1:0:-1], l)
        self.time = l*self.stream_[0].stats.sampling_rate

    def get_time_causal(self): 
        self.time = np.arange(len(self.stream_[0].data))*self.stream_[0].stats.sampling_rate

    def plot_stream(self, subfigure=False, scale_distance=2e5, speed=3900., annotate=False):
        tmp = len(self.stream_)
        i = 0
        if subfigure == True:
            f,axarr = plt.subplots(tmp,1,sharex=True)
            for trace in self.stream_:
                label_ = self.get_info_from_sac(trace)
                try:
                    axarr[i].plot(self.causaltime, trace.data, label=label_, color='black')
                except:
                    axarr[i].plot(self.time, trace.data, label=label_, color='black')
                axarr[i].legend(loc = 2, fontsize = "x-small", borderaxespad=0.)
                axarr[i].yaxis.set_major_formatter(plt.NullFormatter())
                i+=1
            plt.subplots_adjust(wspace=0, hspace=0)
            axarr[0].set_title(str(tmp)+' noise correllations')
            axarr[-1].set_xlabel('Time [s]')

        else:
            f,axarr = plt.subplots(1,1)
            f.hold(True)
            for trace in self.stream_:
                label_ = self.get_info_from_sac(trace)
                try:
                    axarr.plot(self.causaltime, trace.data*scale_distance+trace.stats.sac.dist, label=label_, color='black')
                    if annotate == True:
                        axarr.annotate(label_, xy = (self.causaltime[0], trace.stats.sac.dist+scale_distance))
                except:
                    axarr.plot(self.time, trace.data*scale_distance+trace.stats.sac.dist, label=label_, color='black')
                    if annotate == True:
                        axarr.annotate(label_, xy = (self.time[0], trace.stats.sac.dist+scale_distance))
            if speed != None:
                axarr.plot(self.time[self.time>0], self.time[self.time>0]*speed, 'r--', linewidth =3)
            plt.subplots_adjust(wspace=0, hspace=0)
            axarr.set_title(str(tmp)+' noise correllations')
            axarr.set_xlabel('Time [s]')
            plt.xlim(0, max(self.time))
            plt.ylim(-10e1, 1e7)
        plt.show()

    def get_info_from_sac(self, trace):
        _id = str(trace.id)
        stat = trace.stats
        s1   = _id + '--'
        d1   = ' '+str(stat.sac.dist).strip()+' '
        s2   = stat.sac.kevnm.strip()+'.'+stat.sac.kuser0.strip()+'.'+\
                stat.sac.kuser1.strip()+'.'+ (stat.sac.kuser2).strip()
        return s1+s2


if __name__ == '__main__': 
    pass
