import numpy as np
import matplotlib
matplotlib.use('TkAgg')
import matplotlib.pyplot as plt

from matplotlib.transforms import Bbox, TransformedBbox, \
    blended_transform_factory

from mpl_toolkits.axes_grid1.inset_locator import BboxPatch, BboxConnector,\
    BboxConnectorPatch


def connect_bbox(bbox1, bbox2,
                 loc1a, loc2a, loc1b, loc2b,
                 prop_lines, prop_patches=None):
    if prop_patches is None:
        prop_patches = prop_lines.copy()
        prop_patches["alpha"] = prop_patches.get("alpha", 1)*0.2

    c1 = BboxConnector(bbox1, bbox2, loc1=loc1a, loc2=loc2a, **prop_lines)
    c1.set_clip_on(False)
    c2 = BboxConnector(bbox1, bbox2, loc1=loc1b, loc2=loc2b, **prop_lines)
    c2.set_clip_on(False)

    bbox_patch1 = BboxPatch(bbox1, **prop_patches)
    bbox_patch2 = BboxPatch(bbox2, **prop_patches)

    p = BboxConnectorPatch(bbox1, bbox2,
                           # loc1a=3, loc2a=2, loc1b=4, loc2b=1,
                           loc1a=loc1a, loc2a=loc2a, loc1b=loc1b, loc2b=loc2b,
                           **prop_patches)
    p.set_clip_on(False)

    return c1, c2, bbox_patch1, bbox_patch2, p


def zoom_effect01(ax1, ax2, xmin, xmax, **kwargs):
    """
    ax1 : the main axes
    ax1 : the zoomed axes
    (xmin,xmax) : the limits of the colored area in both plot axes.

    connect ax1 & ax2. The x-range of (xmin, xmax) in both axes will
    be marked.  The keywords parameters will be used ti create
    patches.

    """

    trans1 = blended_transform_factory(ax1.transData, ax1.transAxes)
    trans2 = blended_transform_factory(ax2.transData, ax2.transAxes)

    bbox = Bbox.from_extents(xmin, 0, xmax, 1)

    mybbox1 = TransformedBbox(bbox, trans1)
    mybbox2 = TransformedBbox(bbox, trans2)

    prop_patches = kwargs.copy()
    prop_patches["ec"] = "none"
    prop_patches["alpha"] = 0.2
    prop_patches['color'] = 'grey'

    c1, c2, bbox_patch1, bbox_patch2, p = \
        connect_bbox(mybbox1, mybbox2,
                     loc1a=3, loc2a=2, loc1b=4, loc2b=1,
                     prop_lines=kwargs, prop_patches=prop_patches)

    ax1.add_patch(bbox_patch1)
    ax2.add_patch(bbox_patch2)
    ax2.add_patch(c1)
    ax2.add_patch(c2)
    ax2.add_patch(p)

    return c1, c2, bbox_patch1, bbox_patch2, p


def zoom_effect02(ax1, ax2, **kwargs):
    """
    ax1 : the main axes
    ax1 : the zoomed axes

    Similar to zoom_effect01.  The xmin & xmax will be taken from the
    ax1.viewLim.
    """

    tt = ax1.transScale + (ax1.transLimits + ax2.transAxes)
    trans = blended_transform_factory(ax2.transData, tt)

    mybbox1 = ax1.bbox
    mybbox2 = TransformedBbox(ax1.viewLim, trans)

    prop_patches = kwargs.copy()
    prop_patches["ec"] = "none"
    prop_patches["alpha"] = 0.2
    prop_patches['color'] = 'r'

    c1, c2, bbox_patch1, bbox_patch2, p = \
        connect_bbox(mybbox1, mybbox2,
                     loc1a=3, loc2a=2, loc1b=4, loc2b=1,
                     prop_lines=kwargs, prop_patches=prop_patches)

    ax1.add_patch(bbox_patch1)
    ax2.add_patch(bbox_patch2)
    ax2.add_patch(c1)
    ax2.add_patch(c2)
    ax2.add_patch(p)

    return c1, c2, bbox_patch1, bbox_patch2, p


# import matplotlib.pyplot as plt
#


class Data:
    def __init__(self):
        self.data = np.load('data.npy')
        self.feat = np.load('features.npy')
        # print(self.feat)
        self.name = np.load('names.npy')
        self.reco = np.load('reconstructions2.npy')
        self.rec2 = np.load('reconstructions1.npy')
        self.rena = self.rec2.copy()
        self.rena[0][self.rena[0] == 0] = np.nan

    def plotter(self):
        f, axarr = plt.subplots(1, sharex='col', figsize=(18, 6))
        f.subplots_adjust(hspace=0.03)
        f.subplots_adjust(wspace=0.03)
        # f.hold(True)
        feats = self.feat[0][0]
        # print(feats)
        axarr.plot(self.data,
                   label='Correlation $\eta (t)$:  ' + 'self.name', c='dimgrey',
                   linewidth=2.)

        axarr.plot(self.rena[0],
                   label=('WBFE Waveform $W_0$'),
                   color='black',
                   linewidth=2.)
        # print feats[0][1]
        axarr.annotate('eta', xy=(feats[0] + feats[0] / 2., feats[-1] * feats[3]), xycoords='data',
                       xytext=(-50, 20), textcoords='offset points',
                       arrowprops=dict(arrowstyle='->'), size=10)

        axarr.annotate('t1', xy=(feats[0], 0), xycoords='data',
                       xytext=(-20, -50), textcoords='offset points',
                       arrowprops=dict(arrowstyle='->', lw=2),
                       size=10)

        plt.show()


def do_plot():
    print('Creating the plot...')

    x0 = 1450 #Left bound of first WBFE Iteration subplot
    x1 = 2300 #Right bound of first WBFE Iteration subplot

    x2 = 600 #Left bound of second WBFE Iteration subplot
    x3 = 1200 #Right bound of second WBFE Iteration subplot

    swap_subplots = True


    # print('Hello')
    D = Data()
    feats = D.feat[0][1]
    # print(feats)
    # print(feats)
    if feats[3] == 0:
        feats[3] = -1.
    # D.plotter()
    # tv = 4.38588*1e6/3900
    # print tv
    fig = plt.figure(1, figsize=(8,8))
    fig.patch.set_facecolor('none')
    # fig.hold(True)
    # plt.tight_layout()
    if swap_subplots == True:
        ax1 = plt.subplot(222)
    else:
        ax1 = plt.subplots(221)
    ax2 = plt.subplot(212)
    ax1.set_xlim(x0, x1)
    ax1.set_ylim(-0.99,0.99)

    ax1.plot(D.data,
             label=('WBFE Waveform $W_0$'),
             color='grey',
             linewidth=2.)
    ax1.plot(D.rena[0],
             label=('WBFE Waveform $W_0$'),
             color='black',
             linewidth=2.5)
    ax1.set_ylabel('NORMALIZED AMPLITUDE', size=10)
    ax1.axvline(x=feats[0], ymin=0, ymax=1, ls='--', lw=3, c='grey')
    ax1.xaxis.set_major_formatter(plt.NullFormatter())
    # ax1.yaxis.set_major_formatter(plt.NullFormatter())
    ax1.set_yticks([-0.4, 0.4])
    ax1.scatter(feats[0], 0., s=200., color='black', facecolors='none', linewidth=2)

    ax1.scatter(feats[0] + feats[1], 0., s=200., color='black', facecolors='none', linewidth=2)

    ax1.scatter(507, 0.2585, s=200., color='black', linewidth=2,
                facecolors='none')

    ax1.annotate('$A_r, p$', xy=(506, 0.25), xycoords='data',
                 xytext=(10, -0), textcoords='offset points',
                 # arrowprops=dict(arrowstyle='->', lw=2),
                 size=10)
    # ax1.annotate('$\eta$', xy = (feats[0] + feats[1] / 2., feats[-1] * feats[3]), xycoords='data',
    #              size=10)

    ax1.annotate('$t_s$', xy=(feats[0], 0), xycoords='data',
                 xytext=(-5, -25), textcoords='offset points',
                 # arrowprops=dict(arrowstyle='-', lw=2),
                 size=10)
    s = ((feats[0] + feats[1] / 2.))
    s2 = (feats[1] - feats[0]) / 2.
    # ax1.annotate('', xytext=(feats[0], 0.8), xycoords='data', xy=(tv, 0.8),
    #              arrowprops=dict(arrowstyle='<|-|>', lw=2, color='grey'),
    #              textcoords='data', size=10, color='grey')

    # ax1.annotate('$t_s$', xy=(1080,0.5), xycoords='data',
    #              xytext=(-5, -25), textcoords='offset points',
    #              # arrowprops=dict(arrowstyle='-', lw=2),
    #              size=10)

    # ax1.annotate('$t_e$', xy=(feats[0] + feats[1], 0), xycoords='data',
    #              xytext=(-5, -25), textcoords='offset points',
    #              # arrowprops=dict(arrowstyle='->', lw=2),
    #              size=10)

    ax1.annotate('$t_e, t_l$', xy=(feats[0] + feats[1], 0), xycoords='data',
                 xytext=(-5, -25), textcoords='offset points',
                 # arrowprops=dict(arrowstyle='->', lw=2),
                 size=10)

    ax1.annotate('$\zeta$', xy=(581, 0.03), xycoords='data',
                 xytext=(-20, -0), textcoords='offset points',
                 # arrowprops=dict(arrowstyle='->', lw=2),
                 size=10)

    ax1.annotate('$F, R$', xy=(439,0.1), xycoords='data',
                 xytext=(-20, -0), textcoords='offset points',
                 # arrowprops=dict(arrowstyle='->', lw=2),
                 size=10)
    ax1.set_title('WBFE Iteration 2', size=10)
    ax1.fill_between(np.arange(len(D.data)), D.data, D.rena[0], alpha=0.9, facecolor='white', hatch='////')
    ax1.tick_params(axis='x', labelsize=10)
    ax1.tick_params(axis='y', labelsize=10)

    # ax2.plot(D.data)
    ax2.set_xlim(x2, x3)
    # ax1.plot()
    zoom_effect01(ax1, ax2, x0, x1)


    feats = D.feat[0][0]
    # print(feats)
    if feats[3] == 0:
        feats[3] = -1.
    if swap_subplots == True:
        ax1 = plt.subplot(221)
    else:
        ax1 = plt.subplot(222)
    ax1.plot(D.data, color='grey',
             linewidth=2.)

    ax1.plot(D.rena[0],
             label='WBFE Waveform $W_0$',
             color='black', linewidth=2.5)

    ax1.scatter(feats[0], 0., s=200., color='black', facecolors='none', linewidth=2)
    # ax1.axvline(x=feats[0], ymin=0, ymax=1, ls='--', lw=3, c='grey')
    ax1.scatter(feats[0] + feats[1], 0., s=200., color='black', facecolors='none', linewidth=2)

    ax1.scatter(feats[0] + feats[1] / 2, feats[-1] * feats[3], s=200., color='black', linewidth=2,
                  facecolors='none')

    ax1.annotate('$A_r, p$', xy=(feats[0] + feats[1] / 2., feats[-1] * feats[3]), xycoords='data',
                   xytext=(20, -10), textcoords='offset points',
                   # arrowprops=dict(arrowstyle='->', lw=2),
                   size=10)
    # ax1.annotate('$\eta$', xy = (feats[0] + feats[1] / 2., feats[-1] * feats[3]), xycoords='data',
    #              size=10)

    ax1.annotate('$t_s$', xy=(feats[0], 0), xycoords='data',
                   xytext=(-5, -25), textcoords='offset points',
                   # arrowprops=dict(arrowstyle='-', lw=2),
                   size=10)
    s = ((feats[0] + feats[1] / 2.))
    s2 = (feats[1]-feats[0])/2.
    # ax1.annotate('', xytext = (feats[0],0.8), xycoords = 'data', xy=(tv, 0.8),
    #              arrowprops=dict(arrowstyle='<|-|>', lw=2, color='grey'),
    #              textcoords='data', size=10, color='grey')

    # ax1.annotate('$t_e$', xy=(feats[0] + feats[1], 0), xycoords='data',
    #                xytext=(-5, -25), textcoords='offset points',
    #                # arrowprops=dict(arrowstyle='->', lw=2),
    #                size=10)

    ax1.annotate('$t_e, t_l$', xy=(feats[0] + feats[1], 0), xycoords='data',
                 xytext=(-5, -25), textcoords='offset points',
                 # arrowprops=dict(arrowstyle='->', lw=2),
                 size=10)

    ax1.annotate('$\zeta$', xy=(1196,0.03), xycoords='data',
                 xytext=(-20, -0), textcoords='offset points',
                 # arrowprops=dict(arrowstyle='->', lw=2),
                 size=10)

    ax1.annotate('$F, R$', xy=(1096, 0.25), xycoords='data',
                 xytext=(-20, -0), textcoords='offset points',
                 # arrowprops=dict(arrowstyle='->', lw=2),
                 size=10)

    ax1.annotate('$r_t$', xy=(1100, 0.9), xycoords='data',
                 xytext=(-20, -0), textcoords='offset points',
                 # arrowprops=dict(arrowstyle='->', lw=2),
                 size=10)
    ax1.xaxis.set_major_formatter(plt.NullFormatter())
    ax1.set_yticks([-1.0, 1.0])
    # ax1.yaxis.set_major_formatter(plt.NullFormatter())

    ax1.fill_between(np.arange(len(D.data)), D.data, D.rena[0], alpha=0.9, facecolor='white', hatch='////')

    ax1.set_title('WBFE Iteration 1', size=10)
    ax2.set_title('Noise Correlation with extracted wavelets for  ' +str(D.name), size=10)
    ax1.set_ylim(-1., 1.2)
    ax2.set_ylim(-1., 1.2)
    ax1.set_xlim(x2, x3)
    ax2.set_xlim(0, 2700)
    ax2.plot(D.data, color='grey',
             linewidth=2.)
    ax2.plot(D.rena[0],
             label='WBFE Waveform $W_0$',
             color='black', linewidth=2.5)
    # ax2.axvline(x=tv, ymin=0, ymax=1, ls='--', lw=3, c='grey')
    # ax1.axvline(x=tv, ymin=0, ymax=1, ls='--', lw=3, c='grey')
    ax1.axvline(x=feats[0], ymin=0, ymax=1, ls='--', lw=3, c='grey')
    ax2.set_xlabel('LAG TIME [s]',size=10)
    ax2.set_ylabel('NORMALIZED AMPLITUDE', size=10)
    zoom_effect01(ax1, ax2, x2, x3)
    ax1.tick_params(axis='x', labelsize=10)
    ax1.tick_params(axis='y', labelsize=10)
    ax2.tick_params(axis='x', labelsize=10)
    ax2.tick_params(axis='y', labelsize=10)
    ax2.set_xticks([0,x0,x1,x2,x3,2000,2700])
    # plt.tight_layout()
    plt.show()

if __name__ == '__main__':
    do_plot()
